//
//  ListaRecredenciamento.swift
//  TrabalhoPraticoLP
//
//  Created by Sandor ferreira da silva on 25/11/18.
//  Copyright © 2018 Sandor Ferreira da Silva. All rights reserved.
//

import Foundation

class ListaRecredenciamento {
    
    private let dataHoje: String
    public var dataToShow = [[String]]()
    
    init(_ dataHoje: String) {
        self.dataHoje = dataHoje
    }
    
    public func createRecredenciamentoFile(docentes: [Docente], publicacoes: [Publicacao], regra: Regra, listaQualis: [Qualis]) -> [[String]] {
        var data = [String]()
        let sortedDocentes = docentes.sorted(by: <)
        
        for docente in sortedDocentes {
            //print(docente.Nome)
            var pontos = 0.0;
            var especificacao = "Não"
            
            for publicacao in docente.getPublicacoes(publicacoes: publicacoes) {
                if Int(regra.getAno())! - publicacao.getAno() <= regra.getQtdAnos() {
                    if publicacao.periodico() {
                        pontos += publicacao.getVeiculo().getPontuacao(regra: regra, listaQualis: listaQualis) * regra.getMultiplicador();
                    } else {
                        pontos += publicacao.getVeiculo().getPontuacao(regra: regra, listaQualis: listaQualis)
                    }
                }
            }
            
            if pontos >= regra.getMinimo() {
                especificacao = "Sim"
            }
            
            //print("\(diffDate(date1: docente.DataDeIngresso, date2: dataHoje)) data de ingresso")
            //print("\(diffDate(date1: docente.DataDeNascimento, date2: dataHoje)) data de nascimento")
            
            if diffDate(date1: docente.DataDeNascimento, date2: dataHoje) >= 60 {
                especificacao = "PPS"
            }
            
            if diffDate(date1: docente.DataDeIngresso, date2: dataHoje) <= 3 {
                especificacao = "PPJ"
            }
            
            if docente.isCoordenador {
                especificacao = "Coordenador"
            }
            
            let string = "\(docente.Nome);\(pontos);\(especificacao)\n"
            let content = ["\(docente.Nome)","\(pontos)","\(especificacao)"]
            dataToShow.append(content)
            //print(string)
            
            data.append(string)
        }
        
        createCSVFile(from: data)
        return self.dataToShow
    }
 
    private func createCSVFile(from data: [String]) {
        
        let fileManager = FileManager.default
        
        do {
            let path = try fileManager.url(for: .documentDirectory, in: .allDomainsMask, appropriateFor: nil, create: false)
            let fileURL = path.appendingPathComponent("recredenciamento.csv")
            for content in data {
                try content.write(to: fileURL, atomically: true, encoding: .utf8)
            }
        } catch {
            print("error while trying to create CSV File")
        }
    }
    
    public func diffDate(date1: String, date2: String) -> Int {
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "dd/MM/yyyy"
        let date1_formatted = dateFormatter.date(from: date1)
        let date2_formatted = dateFormatter.date(from: date2)

        let calendar = Calendar.current
        return calendar.dateComponents([.year], from: date1_formatted!, to: date2_formatted!).year!
    }
}

