//
//  ViewController.swift
//  TrabalhoPraticoLP
//
//  Created by Sandor ferreira da silva on 08/11/18.
//  Copyright © 2018 Sandor Ferreira da Silva. All rights reserved.
//

import UIKit
import SwiftCSVExport

class ViewController: UIViewController {

    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    
    @IBAction func createRelatorioUm(_ sender: Any) {
        let regra = getRegras()
        let docentes = getDocentes()
        let listaQualis = getQualis()
        let publicacoes =  getPublicacoes()
        
        let recredenciamento = ListaRecredenciamento("01/01/2017")
        let data = recredenciamento.createRecredenciamentoFile(docentes: docentes, publicacoes: publicacoes, regra: regra, listaQualis: listaQualis)
        
        let relatorioUmController = self.storyboard?.instantiateViewController(withIdentifier: "RelatorioUm") as! Relato_rioUmViewController
        relatorioUmController.dataContent = data
        self.present(relatorioUmController, animated: true, completion: nil)
        
    }
    
    @IBAction func createRelatorioDois(_ sender: Any) {
        
//        let regra = getRegras()
//        let docentes = getDocentes()
//        let veiculos = getVeiculos()
        let listaQualis = getQualis()
        let publicacoes =  getPublicacoes()
        
        let estatisticas = ListaEstatisticas()
        let data = estatisticas.createListaEstatisticas(listaQualis: listaQualis, publicacoes: publicacoes)
        print(data)
        let relatorioDoisController = self.storyboard?.instantiateViewController(withIdentifier: "RelatorioDois") as! Relato_rioDoisViewController
        relatorioDoisController.dataContent = data
        self.present(relatorioDoisController, animated: true, completion: nil)
    }
    
    func getRegras() -> Regra {
        var data = readDataFromCSV(fileName: "regras", fileType: "csv")
        data = cleanRows(file: data!)
        let csvRows = csv(data: data!)
        let regrafromCSV = csvRows[1]
        
        // criando regra
        var listaPontos = [Int]()
        for ponto in regrafromCSV[3].split(separator: ",") {
            listaPontos.append(Int(ponto)!)
        }
        let mul = Double(regrafromCSV[4].replacingOccurrences(of: ",", with: "."))!
        
        let regra = Regra(dataInicio: regrafromCSV[0], dataFim: regrafromCSV[1], multiplicador: mul, qtdAnos: Int(regrafromCSV[5])!, minimoRecredenciamento: Double(regrafromCSV[6])!)
        regra.setQualisPontos(pontosString: regrafromCSV[3], quaStrings: regrafromCSV[2])
        
        return regra
    }
    
    func getDocentes() -> [Docente] {
        var data = readDataFromCSV(fileName: "docentes", fileType: "csv")
        data = cleanRows(file: data!)
        let docentesCSV = csv(data: data!)
        //print(docentesCSV)
        
        var docentes = [Docente]()
        
        for (i,doc) in docentesCSV.enumerated() {
            if i > 0 && doc.count > 1 {
                let codigo = doc[0]
                let nome = doc[1]
                let birth = doc[2]
                let ingresso = doc[3]
                let coordenador = doc[4].isEmpty ? false : true
                
                let docente = Docente(codigo: codigo, nome: nome, dataDeNascimento: birth, dataDeIngresso: ingresso, coordenador: coordenador)
                docentes.append(docente)
            }
        }
        return docentes
    }
    
    func getVeiculos() -> [Veiculo] {
        var data = readDataFromCSV(fileName: "veiculos", fileType: "csv")
        data = cleanRows(file: data!)
        let veiculosCSV = csv(data: data!)
        //print(veiculosCSV)
        
        var veiculos = [Veiculo]()
        for (i,v) in veiculosCSV.enumerated() {
            if i > 0 && v.count > 1 {
                let sigla = v[0]
                let nome = v[1]
                let tipo = v[2]
                var impacto: Double = 0.0
                if v[3].contains(",") {
                    impacto = Double(v[3].replacingOccurrences(of: ",", with: "."))!
                } else {
                    impacto = Double(v[3])!
                }
                let issn = v[4]
                if tipo == "P" {
                    let periodico = Periodico(ISSN: issn, fatorDeImpacto: impacto, sigla: sigla, nome: nome)
                    veiculos.append(periodico)
                } else if tipo == "C" {
                    let conferencia = Conferencia(sigla: sigla, nome: nome)
                    veiculos.append(conferencia)
                }
            }
        }
        return veiculos
    }
    
    func getQualis() -> [Qualis] {
        var data = readDataFromCSV(fileName: "qualis", fileType: "csv")
        data = cleanRows(file: data!)
        let qualisCSV = csv(data: data!)
        //print(qualisCSV)
        
        var listaQualis = [Qualis]()
        for (i, q) in qualisCSV.enumerated() {
            if i > 0 && q.count > 1 {
                let ano = q[0]
                let siglaVeiculo = q[1]
                let qualisname = q[2]
                let qualis = Qualis(ano: ano, siglaVeiculo: siglaVeiculo, qualis: qualisname)
                listaQualis.append(qualis)
            }
        }
        return listaQualis
    }
    
    func getPublicacoes() -> [Publicacao] {
        var data = readDataFromCSV(fileName: "publicacoes", fileType: "csv")
        data = cleanRows(file: data!)
        let publicacoesCSV = csv(data: data!)
        //print(publicacoesCSV)
        
        var publicacoes = [Publicacao]()
        
        for (i, p) in publicacoesCSV.enumerated() {
            if i>0 && p.count > 1 {
                let ano = p[0]
                let siglaVeiculo = p[1]
                let titulo = p[2]
                let codigos = p[3].components(separatedBy: ",")
                let numero = Int(p[4])!
                let volume = p[5].isEmpty ? nil : Int(p[5])!
                let local = p[6].isEmpty ? nil : p[6]
                let pgInicial = p[7]
                let pgFinal = p[8]
                for v in getVeiculos() {
                    if v.Sigla == siglaVeiculo {
                        if local != nil {
                            (v as! Conferencia).setLocal(local: local!)
                        }
                        let publicacao = Publicacao(titulo: titulo, ano: Int(ano)!, pagInicial: Int(pgInicial)!, pagFinal: Int(pgFinal)!, numero: numero, volume: volume, local: local, veiculo: v)
                        publicacao.setAutores(codigos: codigos, docentes: getDocentes())
                        publicacoes.append(publicacao)
                    }
                }
                
            }
        }
        return publicacoes
    }
    
    
    
    func readDataFromCSV(fileName:String, fileType: String)-> String!{
        guard let filepath = Bundle.main.path(forResource: fileName, ofType: fileType)
            else {
                return nil
        }
        do {
            var contents = try String(contentsOfFile: filepath, encoding: .utf8)
            contents = cleanRows(file: contents)
            return contents
        } catch {
            print("File Read Error for file \(filepath)")
            return nil
        }
    }
    
    
    func cleanRows(file:String)->String{
        var cleanFile = file
        cleanFile = cleanFile.replacingOccurrences(of: "\r", with: "\n")
        cleanFile = cleanFile.replacingOccurrences(of: "\n\n", with: "\n")
        return cleanFile
    }

    func csv(data: String) -> [[String]] {
        var result: [[String]] = []
        let rows = data.components(separatedBy: "\n")
        for row in rows {
            let columns = row.components(separatedBy: ";")
            result.append(columns)
        }
        return result
    }
}

